import sys


class ToObjectLogger(object):
    """
    On instantiation, logs system out messages to the self.log list.
    """

    def __init__(self):
        self.terminal = sys.stdout
        self.log = []
        sys.stdout = self

    def write(self, message):
        self.terminal.write(message)
        if message.strip():
            self.log.append(message.strip())

    def flush(self):
        # this flush method is needed for python 3 compatibility.
        # this handles the flush command by doing nothing.
        # you might want to specify some extra behavior here.
        pass


class GetKubermagicInput(Exception):
    pass


class KubermagicAction(object):

    def __init__(self):
        super().__init__()
        self._state = {}
        self._callback = None
        self._logger = ToObjectLogger()

    def _run_callback(self, *args, **kwargs):
        self._callback(*args, **kwargs)

    def run(self):
        raise Exception("run() method not implemented")

    def test_run(self):
        try:
            self.run()
        except GetKubermagicInput as e:
            print(f"Kubernetic Action Triggered {e}")
            print(self._state)

    def notification(self, title, subtitle="", informative_text="", callback=None):
        self._state = {
            "action": "notification",
            "title": title,
            "subtitle": subtitle,
            "informative_text": informative_text,
        }
        self._callback = callback
        raise GetKubermagicInput("Notification")

    def ok_dialog(self, title, dialog_text, ok_button_text=None, callback=None):
        self._state = {
            "action": "ok_cancel_dialog",
            "title": title,
            "dialog_text": dialog_text,
            "ok_button_text": ok_button_text
        }
        self._callback = callback
        raise GetKubermagicInput("OKDialog")

    def ok_cancel_dialog(
        self, title, dialog_text,
        ok_button_text="OK", cancel_button_text="Cancel", callback=None
    ):
        self._state = {
            "action": "ok_cancel_dialog",
            "title": title,
            "dialog_text": dialog_text,
            "ok_button_text": ok_button_text,
            "cancel_button_text": cancel_button_text
        }
        self._callback = callback
        raise GetKubermagicInput("OKCancelDialog")

    def form_dialog(
        self, title, dialog_text, form,
        ok_button_text="OK", cancel_button_text="Cancel", callback=None
    ):

        self._state = {
            "action": "form_dialog",
            "title": title,
            "dialog_text": dialog_text,
            "ok_button_text": ok_button_text,
            "cancel_button_text": cancel_button_text
        }
        self.form = form
        self._callback = callback
        raise GetKubermagicInput("FormDialog")
